@@ -46,8 +46,8 @@ if(rdfCanonizeNative) {
 
 const _TEST_SUITE_PATHS = [
   process.env.TEST_DIR,
-  '../normalization/tests',
-  './test-suites/normalization/tests',
+  '../rdf-dataset-canonicalization/tests',
+  './test-suites/rdf-dataset-canonicalization/tests',
 ];
 const TEST_SUITE = _TEST_SUITE_PATHS.find(pathExists);
 if(!TEST_SUITE) {
@@ -177,7 +177,7 @@ function addTest(manifest, test) {
   }
 
   // expand @id and input base
-  const test_id = test['@id'] || test['id'];
+  const test_id = test['@id'] || test.id;
   test['@id'] = manifest.baseIri + basename(manifest.filename) + test_id;
   test.base = manifest.baseIri + test.input;
   test.manifest = manifest;
