@@ -1,5 +1,5 @@
-/*
- * Copyright (c) 2016-2021 Digital Bazaar, Inc. All rights reserved.
+/*!
+ * Copyright (c) 2016-2022 Digital Bazaar, Inc. All rights reserved.
  */
 'use strict';
 
@@ -200,18 +200,16 @@ module.exports = class NQuads {
   }
 
   /**
-   * Converts an RDF quad to an N-Quad string (a single quad).
+   * Converts RDF quad components to an N-Quad string (a single quad).
    *
-   * @param quad the RDF quad convert.
+   * @param {Object} s - N-Quad subject component.
+   * @param {Object} p - N-Quad predicate component.
+   * @param {Object} o - N-Quad object component.
+   * @param {Object} g - N-Quad graph component.
    *
-   * @return the N-Quad string.
+   * @return {string} the N-Quad.
    */
-  static serializeQuad(quad) {
-    const s = quad.subject;
-    const p = quad.predicate;
-    const o = quad.object;
-    const g = quad.graph;
-
+  static serializeQuadComponents(s, p, o, g) {
     let nquad = '';
 
     // subject can only be NamedNode or BlankNode
@@ -252,6 +250,18 @@ module.exports = class NQuads {
     return nquad;
   }
 
+  /**
+   * Converts an RDF quad to an N-Quad string (a single quad).
+   *
+   * @param quad the RDF quad convert.
+   *
+   * @return the N-Quad string.
+   */
+  static serializeQuad(quad) {
+    return NQuads.serializeQuadComponents(
+      quad.subject, quad.predicate, quad.object, quad.graph);
+  }
+
   /**
    * Converts a legacy-formatted dataset to an array of quads dataset per
    * http://rdf.js.org/.
