@@ -1,5 +1,5 @@
-/*
- * Copyright (c) 2016-2021 Digital Bazaar, Inc. All rights reserved.
+/*!
+ * Copyright (c) 2016-2022 Digital Bazaar, Inc. All rights reserved.
  */
 'use strict';
 
@@ -9,16 +9,22 @@ const Permuter = require('./Permuter');
 const NQuads = require('./NQuads');
 
 module.exports = class URDNA2015 {
-  constructor() {
+  constructor({
+    createMessageDigest = () => new MessageDigest('sha256'),
+    maxDeepIterations = Infinity
+  } = {}) {
     this.name = 'URDNA2015';
     this.blankNodeInfo = new Map();
     this.canonicalIssuer = new IdentifierIssuer('_:c14n');
-    this.hashAlgorithm = 'sha256';
+    this.createMessageDigest = createMessageDigest;
+    this.maxDeepIterations = maxDeepIterations;
     this.quads = null;
+    this.deepIterations = null;
   }
 
   // 4.4) Normalization Algorithm
   async main(dataset) {
+    this.deepIterations = new Map();
     this.quads = dataset;
 
     // 1) Create the normalization state.
@@ -145,13 +151,15 @@ module.exports = class URDNA2015 {
       // 7.1) Create a copy, quad copy, of quad and replace any existing
       // blank node identifiers using the canonical identifiers
       // previously issued by canonical issuer.
-      // Note: We optimize with shallow copies here.
-      const q = {...quad};
-      q.subject = this._useCanonicalId({component: q.subject});
-      q.object = this._useCanonicalId({component: q.object});
-      q.graph = this._useCanonicalId({component: q.graph});
+      // Note: We optimize away the copy here.
+      const nQuad = NQuads.serializeQuadComponents(
+        this._componentWithCanonicalId(quad.subject),
+        quad.predicate,
+        this._componentWithCanonicalId(quad.object),
+        this._componentWithCanonicalId(quad.graph)
+      );
       // 7.2) Add quad copy to the normalized dataset.
-      normalized.push(NQuads.serializeQuad(q));
+      normalized.push(nQuad);
     }
 
     // sort normalized output
@@ -199,7 +207,7 @@ module.exports = class URDNA2015 {
 
     // 5) Return the hash that results from passing the sorted, joined nquads
     // through the hash algorithm.
-    const md = new MessageDigest(this.hashAlgorithm);
+    const md = this.createMessageDigest();
     for(const nquad of nquads) {
       md.update(nquad);
     }
@@ -224,7 +232,7 @@ module.exports = class URDNA2015 {
 
     // 2) Initialize a string input to the value of position.
     // Note: We use a hash object instead.
-    const md = new MessageDigest(this.hashAlgorithm);
+    const md = this.createMessageDigest();
     md.update(position);
 
     // 3) If position is not g, append <, the value of the predicate in quad,
@@ -243,10 +251,17 @@ module.exports = class URDNA2015 {
 
   // 4.8) Hash N-Degree Quads
   async hashNDegreeQuads(id, issuer) {
+    const deepIterations = this.deepIterations.get(id) || 0;
+    if(deepIterations > this.maxDeepIterations) {
+      throw new Error(
+        `Maximum deep iterations (${this.maxDeepIterations}) exceeded.`);
+    }
+    this.deepIterations.set(id, deepIterations + 1);
+
     // 1) Create a hash to related blank nodes map for storing hashes that
     // identify related blank nodes.
     // Note: 2) and 3) handled within `createHashToRelated`
-    const md = new MessageDigest(this.hashAlgorithm);
+    const md = this.createMessageDigest();
     const hashToRelated = await this.createHashToRelated(id, issuer);
 
     // 4) Create an empty string, data to hash.
@@ -489,9 +504,11 @@ module.exports = class URDNA2015 {
     }
   }
 
-  _useCanonicalId({component}) {
+  // canonical ids for 7.1
+  _componentWithCanonicalId(component) {
     if(component.termType === 'BlankNode' &&
       !component.value.startsWith(this.canonicalIssuer.prefix)) {
+      // create new BlankNode
       return {
         termType: 'BlankNode',
         value: this.canonicalIssuer.getId(component.value)
